@@ -27,6 +27,7 @@
 
 #include <glib/gi18n.h>
 #include <gio/gio.h>
+#include <gdk/gdkx.h>
 
 #include "capplet-util.h"
 
@@ -46,11 +47,7 @@
 
 static gboolean in_change = FALSE;
 
-#if GTK_CHECK_VERSION (3, 0, 0)
 static void sample_draw(GtkWidget* darea, cairo_t* cr)
-#else
-static void sample_expose(GtkWidget* darea, GdkEventExpose* expose)
-#endif
 {
 	cairo_surface_t* surface = g_object_get_data(G_OBJECT(darea), "sample-surface");
 	GtkAllocation allocation;
@@ -62,10 +59,6 @@ static void sample_expose(GtkWidget* darea, GdkEventExpose* expose)
 	w = cairo_image_surface_get_width (surface);
 	h = cairo_image_surface_get_height (surface);
 
-#if !GTK_CHECK_VERSION (3, 0, 0)
-	cairo_t *cr = gdk_cairo_create (expose->window);
-#endif
-
 	cairo_set_line_width (cr, 1);
 	cairo_set_line_cap (cr, CAIRO_LINE_CAP_SQUARE);
 
@@ -78,10 +71,6 @@ static void sample_expose(GtkWidget* darea, GdkEventExpose* expose)
 	cairo_set_source_surface (cr, surface, (x - w) / 2, (y - h) / 2);
 
 	cairo_paint(cr);
-
-#if !GTK_CHECK_VERSION (3, 0, 0)
-	cairo_destroy (cr);
-#endif
 }
 
 typedef enum {
@@ -188,11 +177,7 @@ static void setup_font_sample(GtkWidget* darea, Antialiasing antialiasing, Hinti
 	g_object_set_data_full(G_OBJECT(darea), "sample-surface", surface, (GDestroyNotify) cairo_surface_destroy);
 
 	gtk_widget_set_size_request (GTK_WIDGET(darea), width + 2, height + 2);
-#if GTK_CHECK_VERSION (3, 0, 0)
 	g_signal_connect(darea, "draw", G_CALLBACK(sample_draw), NULL);
-#else
-	g_signal_connect(darea, "expose_event", G_CALLBACK(sample_expose), NULL);
-#endif
 }
 
 /*
@@ -435,17 +420,18 @@ dpi_from_pixels_and_mm (int pixels, int mm)
 static double
 get_dpi_from_x_server (void)
 {
-  GdkScreen *screen;
+  GdkScreen  *screen;
   double dpi;
 
   screen = gdk_screen_get_default ();
+
   if (screen) {
     double width_dpi, height_dpi;
 
-    width_dpi = dpi_from_pixels_and_mm (gdk_screen_get_width (screen),
-					gdk_screen_get_width_mm (screen));
-    height_dpi = dpi_from_pixels_and_mm (gdk_screen_get_height (screen),
-					 gdk_screen_get_height_mm (screen));
+    Screen *xscreen = gdk_x11_screen_get_xscreen (screen);
+
+    width_dpi = dpi_from_pixels_and_mm (WidthOfScreen (xscreen), WidthMMOfScreen (xscreen));
+    height_dpi = dpi_from_pixels_and_mm (HeightOfScreen (xscreen), HeightMMOfScreen (xscreen));
 
     if (width_dpi < DPI_LOW_REASONABLE_VALUE || width_dpi > DPI_HIGH_REASONABLE_VALUE ||
         height_dpi < DPI_LOW_REASONABLE_VALUE || height_dpi > DPI_HIGH_REASONABLE_VALUE)
@@ -467,16 +453,19 @@ static void
 dpi_load (GSettings     *settings,
 	  GtkSpinButton *spinner)
 {
-  gdouble value = g_settings_get_double (settings, FONT_DPI_KEY);
+  GdkScreen *screen;
+  gint scale;
   gdouble dpi;
 
-  if (value != 0)
-    dpi = value;
-  else
+  screen = gdk_screen_get_default ();
+  scale = gdk_window_get_scale_factor (gdk_screen_get_root_window (screen));
+  dpi = g_settings_get_double (settings, FONT_DPI_KEY);
+
+  if (dpi == 0)
     dpi = get_dpi_from_x_server ();
 
-  if (dpi < DPI_LOW_REASONABLE_VALUE)
-    dpi = DPI_LOW_REASONABLE_VALUE;
+  dpi *= (double)scale;
+  dpi = CLAMP(dpi, DPI_LOW_REASONABLE_VALUE, DPI_HIGH_REASONABLE_VALUE);
 
   in_change = TRUE;
   gtk_spin_button_set_value (spinner, dpi);
@@ -491,6 +480,15 @@ dpi_changed (GSettings *settings,
   dpi_load (settings, user_data);
 }
 
+static void
+monitors_changed (GdkScreen      *screen,
+		  AppearanceData *data)
+{
+  GtkWidget *widget;
+  widget = appearance_capplet_get_widget (data, "dpi_spinner");
+  dpi_load (data->font_settings, GTK_SPIN_BUTTON (widget));
+}
+
 static void
 dpi_value_changed (GtkSpinButton *spinner,
 		   GSettings     *settings)
@@ -504,7 +502,13 @@ dpi_value_changed (GtkSpinButton *spinner,
    * received from GSettings, this may cause mildly strange effects.
    */
   if (!in_change) {
-    gdouble new_dpi = gtk_spin_button_get_value (spinner);
+    GdkScreen *screen;
+    gint scale;
+    gdouble new_dpi;
+
+    screen = gdk_screen_get_default ();
+    scale = gdk_window_get_scale_factor (gdk_screen_get_root_window (screen));
+    new_dpi = gtk_spin_button_get_value (spinner) / (double)scale;
 
     g_settings_set_double (settings, FONT_DPI_KEY, new_dpi);
 
@@ -550,6 +554,9 @@ cb_show_details (GtkWidget *button,
 
     g_signal_connect (data->font_settings, "changed::" FONT_DPI_KEY, G_CALLBACK (dpi_changed), widget);
 
+    /* Update font DPI when window scaling factor is changed */
+    g_signal_connect (gdk_screen_get_default (), "monitors-changed", G_CALLBACK (monitors_changed), data);
+
     setup_font_sample (appearance_capplet_get_widget (data, "antialias_none_sample"),      ANTIALIAS_NONE,      HINT_SLIGHT);
     setup_font_sample (appearance_capplet_get_widget (data, "antialias_grayscale_sample"), ANTIALIAS_GRAYSCALE, HINT_SLIGHT);
     setup_font_sample (appearance_capplet_get_widget (data, "antialias_subpixel_sample"),  ANTIALIAS_RGBA,      HINT_SLIGHT);
